
/*************************************************************************/
/* THIS VERSION: 8/2/13                                                 **/
/* COMPUTE CSB(P) STATISTICS IN:
   M.H.PESARAN, V.L.Smith, T.Yamagata              			**/
/* " Panel Unit Root Tests in the Presence of a Multifactor Error
    Structure"                                                          **/
/* NOTE:                                                                **/
/* Please report any problems you might have to: lvs21@cam.ac.uk      	**/
/*                                                                      **/
/* {CSB_p_mat,CSBi_p_mat} = CSB(y_mat,xbar_mat,maxp,case)        	**/

/* <<INPUT>>                                                            **/
/* y_mat (T x N) matrix                                                 **/
/* xbar_mat (T x K) matrix of cross-section averages                    **/
/*    where K is the number of variables in xbar_mat                    **/
/* N        : number of cross section dimension                     	**/
/* T		: number of observations for i's                       	**/
/* maxp (more than or equal to zero) : maximum number of (CSBi)lag order**/

/* case=2: with intercept                                               **/
/* case=3: with intercept and trend                                     **/
/* <<OUTPUT>>                                                          **/
/* CSB_p_mat: (maxp+1) x 1 vector of CSB(p) statistics      		**/
/*          for p=0,1,...,maxp in ascending order                       **/
/*  The CSB(p) statistic is defined as simple average of the           **/
/*  CSB(p)_i statistics for p=0,1,...,maxp in ascending order          **/
/*                                                                      **/   
/* CSBi_p_mat: a N x (maxp+1) matrix, reports all CSB(p)_i statistics  **/   
/*              concatenated vertically, from the top p=0,p=1,,,p=maxp  **/    
/*                                                                      **/   
/* NB: Appropriate critical values for the CSB statistic are found      **/
/*  in Tables B3-B4 in:
   M.H.PESARAN, V.L.Smith, T.Yamagata (2013),      			**/
/* " Panel Unit Root Tests in the Presence of a Multifactor Error
    Structure"                                                          **/	
/* L. Vanessa Smith, 08 Feburary 2013                                   **/
/*                                                                      **/
/*************************************************************************/


proc(2)=CSB(y_mat,xbar_mat,maxp,case);/* case=2;intercept, case=3 intercept and trend */
local i,n,y,y_1,dy,c_p,dyp,s,s2,tstar,x,xx,phi,e,se,SBstat,xr,phir,er,
CSB_p_mat,CSBi_p_mat,sr2,xbar,zbar, dzbar, dzbarp,k;

n=cols(y_mat);
if xbar_mat==0;k=0;
zbar=meanc(y_mat');
zbar=zbar-zbar[1,.];
dzbar=zbar-lag(zbar);
dzbar=trim(dzbar,1,0);
dzbarp=plag(dzbar,maxp);

else;

xbar=xbar_mat;
k=cols(xbar);
zbar=meanc(y_mat')~xbar;
zbar=zbar-zbar[1,.];
dzbar=zbar-lag(zbar);
dzbar=trim(dzbar,1,0);
dzbarp=plag(dzbar,maxp);

endif;

CSBi_p_mat={};

i=1;
do while i<=n;
y=y_mat[.,i]-y_mat[1,i];/** deviation from the initial values **/

y_1=lag(y);
dy=y-y_1;

c_p=1;dyp=dy;
do while c_p<=maxp;
if c_p==1;dyp=lag(dyp);
else; dyp=dyp~lag(dyp[.,c_p-1]);
endif;
c_p=c_p+1;
endo;
dyp=dyp[maxp+2:rows(dy),.];


c_p=0;
do while c_p<=maxp;

s=dy[maxp+2:rows(dy)];
tstar=rows(s);/**effective # of obs. */

if c_p==0;

    if case==2;

        xr=dzbarp[.,1:(k+1)];
            if tstar<=cols(xr);SBstat=SBstat|_dxmiss;goto label1;
            else;
            phir=s/xr;er=s-xr*phir;sr2=er'er/(tstar-cols(xr));
            endif;

    elseif case==3;

        
        xr=ones(tstar,1)~dzbarp[.,1:(k+1)];
           if tstar<=cols(xr);SBstat=SBstat|_dxmiss;goto label1;
            else;phir=s/xr;er=s-xr*phir;sr2=er'er/(tstar-cols(xr));
		   endif;
	

    elseif (case/=2 or case/=3);
	print "ERROR MESSAGE: Please use case 2 or case 3 only";
    end;	
		endif;

SBstat=sumc(cumsumc(er)^2)/(sr2*rows(er)^2);

label1:

else;

xr=xr~dyp[.,c_p]~dzbarp[.,1+c_p*(k+1):(c_p+1)*(k+1)];

if tstar<=cols(xr);
SBstat=SBstat|_dxmiss;
else;
phir=s/xr;er=s-xr*phir;sr2=er'er/(tstar-cols(xr));
SBstat=SBstat|(sumc(cumsumc(er)^2)/(sr2*rows(er)^2));
endif;

endif;

c_p=c_p+1;
endo;

CSBi_p_mat=CSBi_p_mat|SBstat';
i=i+1;
endo;

CSB_p_mat=meanc(CSBi_p_mat);

retp(CSB_p_mat,CSBi_p_mat);
endp;




proc(1)=plag(y,p);
local yp,c_p;
yp=y[p+1:rows(y),.];
c_p=1;
do while c_p<=p;
yp=yp~y[p+1-c_p:rows(y)-c_p,.];
c_p=c_p+1;
endo;
retp(yp);
endp;


